from enum import StrEnum

class Constant(StrEnum):
    USER_TOKEN = 'user_token'
    AES_DATA = 'aes_data'
    DEFAULT_ENCODING = 'utf-8'
    MESSAGE_LOG_INTERNAL = 'MESSAGE_LOG_INTERNAL'
    DEFAULT_EMPTY_VALUE = '*'
    RESPONSE_TYPE_ERROR = 'ERROR'
    RESPONSE_TYPE_REQUEST = 'REQUEST'
    RESPONSE_TYPE_RESPONSE = 'RESPONSE'
    MESSAGE_LOG_EXTERNAL = 'MESSAGE_LOG_EXTERNAL'