from enum import StrEnum

class Code(StrEnum):
    PROCESS_SUCCESS_CODE = 'DB_0004'
    UNAUTHORIZATED_CODE = 'DB_ERROR_0006'
    REQUEST_NOT_COMPLETED_CODE = 'DB_ERROR_9999'
    DATA_NOT_FOUND_CODE = 'DB_ERROR_0404'
    APP_ERROR_CODE = 'DB_ERROR_APP'
    INVALID_REQUEST_PARAMS_CODE = 'DB_WARNING_INVALID_REQUEST_PARAMS'
    DATABASE_ERROR_CODE = 'DB_WARNING_DATABASE_ERROR'
    RSA_ERROR_CODE = 'DB_ERROR_RSA'
    AES_ERROR_CODE = 'DB_ERROR_AES'
    JWT_ERROR_CODE = 'DB_ERROR_JWT'
    HTTP_ERROR_CODE = 'DB_ERROR_HTTP'
    AZURE_ERROR_CODE = 'DB_ERROR_AZURE'
    REDIS_ERROR_CODE = 'DB_ERROR_REDIS'
    REQUEST_VALIDATION_ERROR_CODE = 'DB_WARNING_REQUEST_VALIDATION'
    MISSING_FIELD_ERROR_CODE = 'DB_WARNING_MISSING_FIELD'
    PROFILE_PERMISSION_DENIED_CODE = 'DB_WARNING_PROFILE_PERMISSION_DENIED'
    INVALID_FORMAT_CODE = 'DB_WARNING_INVALID_FORMAT'